@@ -24,6 +24,13 @@ class Mod9EngineFailedStatusError(Exception):
     pass
 
 
+# Used to allow arbitrary `model`s to be loaded into Engine and requested by user.
+class ObjectContainingEverything:
+    """Object that always returns True to `if x in ObjectContainingEverything()` queries."""
+    def __contains__(self, x):
+        return True
+
+
 class GoogleConfigurationSettingsAndMappings:
     """
     Hold dicts used to translate keys and values from Google to Mod9
@@ -59,8 +66,9 @@ class GoogleConfigurationSettingsAndMappings:
             'word-confidence',
             'word-intervals',
             'transcript-formatted',
-            'transcript-alternatives-max',
-            'phrase-alternatives-max',   # only in Mod9
+            'transcript-alternatives',
+            'phrase-alternatives',   # only in Mod9
+            'model',                 # Mod9-only option
         )
         self.google_allowed_keys = (
             'encoding',
@@ -70,6 +78,7 @@ class GoogleConfigurationSettingsAndMappings:
             'enableAutomaticPunctuation',
             'maxAlternatives',
             'maxPhraseAlternatives',     # only in Mod9
+            'model',                     # Mod9-only option
         )
 
         # Map from (Google key) to (Mod9 key).
@@ -93,6 +102,7 @@ class GoogleConfigurationSettingsAndMappings:
         self.google_punctuation_allowed_values = {True, False}
         self.google_max_alternatives_allowed_values = set(i for i in range(31))
         self.google_max_phrase_alternatives_allowed_values = set(i for i in range(1, 101))
+        self.model_allowed_values = ObjectContainingEverything()
 
         # Group allowed Google values. To be used in following dict.
         self.google_allowed_values = (
@@ -103,6 +113,7 @@ class GoogleConfigurationSettingsAndMappings:
             self.google_punctuation_allowed_values,
             self.google_max_alternatives_allowed_values,
             self.google_max_phrase_alternatives_allowed_values,
+            self.model_allowed_values,
         )
 
         # Map from (Mod9 key) to (allowed Google values) for given key.
@@ -188,7 +199,7 @@ def input_to_mod9(google_input_settings, module):
         if isinstance(mod9_audio_settings, GeneratorType):
             # Split generator so utils.parse_wav_encoding() can look at content header.
             mod9_audio_settings, mod9_audio_settings_clone = itertools.tee(mod9_audio_settings)
-        wav_encoding = utils.parse_wav_encoding(mod9_audio_settings)
+        wav_encoding, wav_sample_rate = utils.parse_wav_encoding(mod9_audio_settings)
         if wav_encoding:
             mod9_config_settings['format'] = 'wav'
             if 'encoding' in mod9_config_settings:
@@ -200,6 +211,13 @@ def input_to_mod9(google_input_settings, module):
                     )
                 # The Mod9 ASR Engine complains if both WAV format and audio encoding are specified.
                 del mod9_config_settings['encoding']
+            if wav_sample_rate:
+                if 'rate' in mod9_config_settings:
+                    if mod9_config_settings['rate'] != wav_sample_rate:
+                        raise ValueError(f"Specified rate, {mod9_config_settings['rate']},"
+                                         f" differs from WAV header rate, {wav_sample_rate}.")
+                else:
+                    mod9_config_settings['rate'] = wav_sample_rate
         else:
             mod9_config_settings['format'] = 'raw'
             if 'encoding' not in mod9_config_settings:
@@ -210,6 +228,15 @@ def input_to_mod9(google_input_settings, module):
         except UnboundLocalError:
             pass
 
+    if 'rate' in mod9_config_settings and 'model' not in mod9_config_settings:
+        # Set model associated with user-passed rate, but don't overwrite a user-passed model.
+        models = utils.find_loaded_models_with_rate(mod9_config_settings['rate'])
+        # Use first English model in list -> first loaded model.
+        for model in models:
+            if model['language'].lower() == 'en-us':
+                mod9_config_settings['model'] = model['name']
+                break
+
     # Mod9 TCP does not accept 'rate' argument for 'wav' format.
     if 'format' not in mod9_config_settings or mod9_config_settings['format'] == 'wav':
         if 'rate' in mod9_config_settings:
@@ -291,14 +318,12 @@ def google_config_settings_to_mod9(google_config_settings):
 
     # Set N-best settings. Always get 1-best or more:
     #  ``result_from_mod9()`` iterates through alternatives.
-    mod9_config_settings['transcript-alternatives'] = True
-    n_best_N = mod9_config_settings.get('transcript-alternatives-max', 1)
+    n_best_N = mod9_config_settings.get('transcript-alternatives', 1)
     # Google sets n_best_N: 0 -> 1.
-    mod9_config_settings['transcript-alternatives-max'] = n_best_N if n_best_N > 0 else 1
+    mod9_config_settings['transcript-alternatives'] = n_best_N if n_best_N > 0 else 1
 
     # This option only applies with speech_mod9.
-    if mod9_config_settings.get('phrase-alternatives-max'):
-        mod9_config_settings['phrase-alternatives'] = True
+    if mod9_config_settings.get('phrase-alternatives'):
         mod9_config_settings['phrase-intervals'] = True
 
     return mod9_config_settings
